using System;
using System.Data;
using System.Data.Common;
using System.Data.OleDb;
using ShopCommonClassLib;


namespace ShopClassLibrary
{
	/// <summary>
	/// Summary description for interface ICustomer.
	/// </summary>
	public interface ICustomer
	{
		DataSet GetByID(string sUser, string sPass, long lClientID);
		DataSet FindByName(string sUser, string sPass, string sNamePattern);
		long AddCustomer(string sUser, string sPass, 
			             string sName, string sAddress, string sPhone, string sFax, string sEMail, out ShopStatusEnum status);
		long AddShopCustomer(string sUser, string sPass, 
			                 string sName, string sAddress, string sPhone, string sFax, string sEMail, out ShopStatusEnum status);

		ShopStatusEnum UpdateCustumerData(string sUser, string sPass, 
			                              long lCustumerID, string sName, string sAddress, string sPhone, string sFax, string sEMail);
	}

	/// <summary>
	/// Summary description for Customer.
	/// </summary>
	public class Customer : ICustomer
	{
		public Customer()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		public DataSet GetByID(string sUser, string sPass, long lClientID)
		{
			ShopStatusEnum status = Comum.ValidateUser(sUser, sPass);

			if (status != ShopStatusEnum.OK)
			{
				return null;
			}

			DataSet oSet = Comum.GetByID("Customers", "CustomerID", lClientID);

			return oSet;
		}

		public DataSet FindByName(string sUser, string sPass, string sNamePattern)
		{
			ShopStatusEnum status = Comum.ValidateUser(sUser, sPass);

			if (status != ShopStatusEnum.OK)
			{
				return null;
			}
			
			DataSet oSet = Comum.FindByStringField("Customers", "Name", sNamePattern, false);
	
			return oSet;
		}

		public long AddCustomer(string sUser, string sPass, 
			                    string sName, string sAddress, string sPhone, string sFax, string sEMail, out ShopStatusEnum status)
		{
			//
			// validar o utilizador
			//
			status = Comum.ValidateUser(sUser, sPass);
			if (status != ShopStatusEnum.OK)
			{
				return -1;
			}

			long lCustomerID = -1;
			OleDbConnection oConn = null;

			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				oConn = new OleDbConnection(Comum.sConn);
				oConn.Open();

				//
				// criar comando SQL a executar
				//
				string sSqlCmd = "INSERT INTO Customers (Name, Address, PhoneNb, FaxNb, EMail) " +
					                         "Values    ('" + sName + "', '" + sAddress + "', '" + sPhone + "', '" + sFax + "', '" + sEMail +"')";
				OleDbCommand oCmd = new OleDbCommand(sSqlCmd, oConn);
		
				//
				// executar o comando 
				//
				int insRows = oCmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (insRows == 0)
					// no inseriu registo
					status = ShopStatusEnum.NOT_OK;
				else
				{
					//
					// inseriu registo
					// obter novo cdigo de id gerado pela BD
					//
					OleDbCommand idCmd = new OleDbCommand("SELECT @@IDENTITY", oConn);
					lCustomerID = (int)idCmd.ExecuteScalar();
					status = ShopStatusEnum.OK;		
				}
			}

			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <AddCustomer> 'class Customer': " + ex);
				status = ShopStatusEnum.ERROR;
			}

			finally
			{
				//
				// fechar a conexo
				//
				if (oConn.State == ConnectionState.Open)
					oConn.Close();
			}

			return lCustomerID;
		}

		public long AddShopCustomer(string sUser, string sPass, 
			                        string sName, string sAddress, string sPhone, string sFax, string sEMail, out ShopStatusEnum status)
		{
			//
			// validar o utilizador
			//
			status = Comum.ValidateUser(sUser, sPass);
			if (status != ShopStatusEnum.OK)
			{
				return -1;
			}

			long lCustomerID = -1;

			ShopData shop = new ShopData();
			ShopDataSet dsShop = new ShopDataSet();

			try
			{	
				//
				// criar comando SQL a executar
				//
				ShopDataSet.CustomersRow drCust = dsShop.Customers.AddCustomersRow(sAddress, sEMail, sFax, sName, sPhone);
				
				//
				// abrir a conexo
				//
				shop.ShopConnection.Open();

				//
				// associar mtodo para tratamento de eventos
				//
				shop.ShopDACustomers.RowUpdated += new OleDbRowUpdatedEventHandler(this.Customers_RowUpdated);

				//
				// inserir DataRow na base de dados 
				//
				int insRows = shop.ShopDACustomers.Update(dsShop);

				//
				// verificar o resultado
				//
				if (insRows == 0)
					// no inseriu registo
					status = ShopStatusEnum.NOT_OK;
				else
				{
					//
					// inseriu registo
					// obter novo cdigo de id gerado pela BD
					//
					lCustomerID = drCust.CustomerID;
					status = ShopStatusEnum.OK;		
				}

				dsShop.AcceptChanges();
			}

			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				dsShop.RejectChanges();

				System.Console.WriteLine("EXCEPO no mtodo <AddCustomer> 'class Customer': " + ex);
				status = ShopStatusEnum.ERROR;
			}

			finally
			{
				//
				// fechar a conexo
				//
				shop.ShopConnection.Close();

				//
				// dissociar mtodo para tratamento de eventos
				//
				shop.ShopDACustomers.RowUpdated -= new OleDbRowUpdatedEventHandler(this.Customers_RowUpdated);
			}

			return lCustomerID;
		}

		//
		// mtodo usado para tratar eventos na insero de registos e obter o cdigo (autonumber) gerado
		// adaptado de:
		// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/cpguide/html/cpconRetrievingIdentityOrAutonumberValues.asp
		//
		protected void Customers_RowUpdated(object sender, OleDbRowUpdatedEventArgs args)
		{
			int newID = 0;

			try
			{
				// Include a variable and a command to retrieve the identity value from the Access database.
				OleDbCommand idCMD = new OleDbCommand("SELECT @@IDENTITY", args.Command.Connection);
				idCMD.Transaction = args.Command.Transaction;

				// Retrieve the identity value and store it in the CategoryID column.
				newID = (int)idCMD.ExecuteScalar();
				args.Row["CustomerID"] = newID;
			}

			catch(System.Data.OleDb.OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO 'OLEDB' na 'class Sale': " + ex);
			}

			finally
			{
			}
		}

		public ShopStatusEnum UpdateCustumerData(string sUser, string sPass, 
			                                     long lCustomerID, string sName, string sAddress, string sPhone, string sFax, string sEMail)
		{
			//
			// validar o utilizador
			//
			ShopStatusEnum status = Comum.ValidateUser(sUser, sPass);
			if (status != ShopStatusEnum.OK)
			{
				return status;
			}

			OleDbConnection oConn = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				oConn = new OleDbConnection(Comum.sConn);
				oConn.Open();
				
				//
				// criar comando SQL a executar
				//
				string sSqlCmd = "Update Customers SET Name = '"    + sName +
					                              "', Address = '" + sAddress +
					                              "', PhoneNb = '" + sPhone +
					                              "', FaxNb = '"   + sFax +
					                              "', EMail = '"   + sEMail +
					            "' WHERE CustomerID = " + lCustomerID.ToString(); 
				OleDbCommand oCmd = new OleDbCommand(sSqlCmd, oConn);

				//
				// executar o comando 
				//
				int updRows = oCmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (updRows == 0)
					// no actualizou registo
					status = ShopStatusEnum.NOT_OK;
				else
					// actualizou registo
					status = ShopStatusEnum.OK;
			}

			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <UpdateCustumerData> 'class Customer': " + ex);
				status = ShopStatusEnum.ERROR;
			}

			finally
			{
				//
				// fechar a conexo
				//
				if (oConn.State == ConnectionState.Open)
					oConn.Close();
			}


			return status;
		}
	}
}
