using System;
using System.Data;
using System.Data.Common;
using System.Data.OleDb;

namespace ShopClassLibrary
{
	/// <summary>
	/// Summary description for interface ICustomer.
	/// </summary>
	public interface IProduct
	{
		DataSet GetByID(string sUser, string sPass, long lProductID);
		DataSet FindByName(string sUser, string sPass, string sNamePattern);
		ShopStatusEnum InStock(string sUser, string sPass, long lProductID, long lQuant);
	}
	
	/// <summary>
	/// Summary description for Product.
	/// </summary>
	public class Product : IProduct
	{
		public Product()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		public DataSet GetByID(string sUser, string sPass, long lProductID)
		{
			//
			// validar utilizador
			//
			ShopStatusEnum status = Comum.ValidateUser(sUser, sPass);

			if (status != ShopStatusEnum.OK)
			{
				return null;
			}

			DataSet oSet = Comum.GetByID("Products", "ProductID", lProductID);

			return oSet;
		}

		public DataSet FindByName(string sUser, string sPass, string sNamePattern)
		{
			//
			// validar utilizador
			//
			ShopStatusEnum status = Comum.ValidateUser(sUser, sPass);

			if (status != ShopStatusEnum.OK)
			{
				return null;
			}
			
			DataSet oSet = Comum.FindByStringField("Products", "Description", sNamePattern, false);
	
			return oSet;
		}

		public ShopStatusEnum InStock(string sUser, string sPass, long lProductID, long lQuant)
		{
			//
			// validar utilizador
			//
			ShopStatusEnum status = Comum.ValidateUser(sUser, sPass);

			if (status != ShopStatusEnum.OK)
			{
				return status;
			}

			//
			// verifica a quantidade em stock
			//
			status = InStock(lProductID, lQuant);

			return status;
		}

		static public ShopStatusEnum InStock(long lProductID, long lQuant)
		{
			long lStkQtd = 0;
			OleDbConnection oConn = null;

			ShopStatusEnum status = ShopStatusEnum.OK;

			try
			{	
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				oConn = new OleDbConnection(Comum.sConn);
				oConn.Open();

				//
				// criar comando SQL a executar
				//
				string sSqlCmd = "SELECT StockQtd FROM Products WHERE ProductID = " + lProductID.ToString();
				OleDbCommand oCmd = new OleDbCommand(sSqlCmd, oConn);
		
				//
				// executar o comando 
				//
				OleDbDataReader oDR = oCmd.ExecuteReader();

				//
				// verificar se retornou dados 
				// ( necessrio manter a conexo aberta pois estamos a usar um DataReader)
				//
				if (oDR.HasRows)
				{
					oDR.Read();
					lStkQtd = oDR.GetInt32(0);
					//
					// verifica stock
					//
					if (lStkQtd >= lQuant)
						status = ShopStatusEnum.OK;
					else
						status = ShopStatusEnum.INSUFICIENT_STOCK;
				}
				else
				{
					status = ShopStatusEnum.INVALID_PRODUCT_ID;
				}

				oDR.Close();
			}

			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <StockQuantity> 'class Product': " + ex);
			}

			finally
			{
				//
				// fechar a conexo
				//
				if (oConn.State == ConnectionState.Open)
					oConn.Close();
			}

			return status;
		}
	}
}
