using System;
using System.Data;
using System.Data.Common;
using System.Data.OleDb;
using ShopCommonClassLib;

namespace ShopClassLibrary
{
	/// <summary>
	/// Summary description for interface ICustomer.
	/// </summary>
	public interface ISale
	{
		DataSet CreateSaleDetails(string sUser, string sPass);
		ShopStatusEnum AddSale(string sUser, string sPass, long lCustomerID, System.DateTime dtDate, DataSet dsDetails);

		ShopStatusEnum AddShopSale(string sUser, string sPass, long lCustomerID, System.DateTime dtDate, ShopDataSet dsShop);
	}
	
	/// <summary>
	/// Summary description for Sale.
	/// </summary>
	public class Sale : ISale
	{
		public Sale()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		public DataSet CreateSaleDetails(string sUser, string sPass)
		{
			//
			// validar utilizador
			//
			ShopStatusEnum status = Comum.ValidateUser(sUser, sPass);

			if (status != ShopStatusEnum.OK)
			{
				return null;
			}
			
			OleDbConnection oConn = null;
			DataSet dsSaleDetails = null;
			OleDbDataAdapter oAdapter = null;

			try
			{
				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				oConn = new OleDbConnection(Comum.sConn);
				oConn.Open();

				//
				// Criar um dataset para o 'Sale Details'
				//
				dsSaleDetails = new DataSet();

				//
				// Cria o 'DataAdapter' e preenche o dataset com o 'schema' da tabela 'SaleDetails'
				// NOTA: no  necessario carregar registos
				//
				oAdapter = new OleDbDataAdapter("SELECT * FROM SaleDetails", oConn);
				oAdapter.FillSchema(dsSaleDetails, SchemaType.Source, "SaleDetails");

				dsSaleDetails.Tables["SaleDetails"].Columns["SaleID"].AllowDBNull = true;
			}

			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no mtodo <CreateSalesDetail> 'class Sale': " + ex);
				dsSaleDetails = null;
			}

			finally
			{
				//
				// fechar a conexo
				//
				oConn.Close();
			}

			return dsSaleDetails;
		}

		public ShopClassLibrary.ShopStatusEnum AddSale(string sUser, string sPass, long lCustomerID, DateTime dtDate, DataSet dsSales)
		{
			//
			// validar utilizador
			//
			ShopStatusEnum status = Comum.ValidateUser(sUser, sPass);

			if (status != ShopStatusEnum.OK)
			{
				return status;
			}

			//
			// verificar se o cliente existe
			//
			status = Comum.CheckIfExistsID("Customers", "CustomerID", lCustomerID);

			if ( status != ShopStatusEnum.OK)
			{
				status = ShopStatusEnum.INVALID_CUSTOMER_ID;
				return status;
			}
			
			OleDbConnection oConn   = null;
			OleDbTransaction oTrans = null;

			string sProdList = "(";

			try
			{
				//
				// verificar se os produtos colocados nas linhas de venda existem e se esto em stock
				//
				foreach(DataRow dr in dsSales.Tables["SaleDetails"].Rows)
				{
					//
					// obter cdigo de produto e quantidade pretendida do recordset
					//
					long lProdID = (int)dr["ProductID"];
					long lQuant  = (int)dr["Quantity"];

					//
					// verificar a existncia do produto em stock
					//
					status = Product.InStock(lProdID, lQuant);
					if (status != ShopStatusEnum.OK)
						return status;

					sProdList += lProdID.ToString() + ",";
				}

				sProdList = sProdList.Substring(0, sProdList.Length-1) + ")";

				//
				// est tudo OK ==> inserir venda
				//
				long lSaleID;
				status = ShopStatusEnum.OK;

				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				oConn = new OleDbConnection(Comum.sConn);
				oConn.Open();

				//
				// iniciar transaco
				//
				oTrans = oConn.BeginTransaction();

				//
				// Inserir a tabela 'Sales' no dataset 'dsSales'
				//
				string sSqlCmd = "SELECT * FROM Sales";
				OleDbDataAdapter daSales = new OleDbDataAdapter(sSqlCmd, oConn);
				daSales.SelectCommand.Transaction = oTrans;
				daSales.FillSchema(dsSales, SchemaType.Source, "Sales");

				//
				// associar mtodo para tratamento de eventos
				//
				daSales.RowUpdated += new OleDbRowUpdatedEventHandler(this.Sales_RowUpdated);

				//
				// Cria uma nova 'DataRow' do tipo 'Sales' e coloca a informao da venda
				//
				DataRow drSale = dsSales.Tables["Sales"].NewRow();
				drSale.BeginEdit();
					drSale["CustomerID"] = lCustomerID;
					drSale["DateOfSale"] = dtDate;
				drSale.EndEdit();
				dsSales.Tables["Sales"].Rows.Add(drSale);

				//
				// Insere a nova venda na base de dados
				//
				OleDbCommandBuilder cbSales = new OleDbCommandBuilder(daSales);
				daSales.InsertCommand = cbSales.GetInsertCommand();
				daSales.Update(dsSales,"Sales");

				//
				// obter novo cdigo de id gerado pela BD
				//
				lSaleID = (int)drSale["SaleID"];

				//
				// associar mtodo para tratamento de eventos
				//
				daSales.RowUpdated -= new OleDbRowUpdatedEventHandler(this.Sales_RowUpdated);

				//
				// obter novo cdigo de id gerado pela BD
				// -> Este cdigo s funciona quando  inserido apenas um registo
				// -> melhor o tratamento de eventos
				// OleDbCommand idCmd = new OleDbCommand("SELECT @@IDENTITY", oConn, oTrans);
				// lSaleID = (int)idCmd.ExecuteScalar();
				//

				//
				// Carregar a tabela 'Products' para o dataset 'dsSales'
				//
				sSqlCmd = "SELECT * FROM Products WHERE ProductID IN " + sProdList;

				OleDbDataAdapter daProducts = new OleDbDataAdapter(sSqlCmd, oConn);
				daProducts.SelectCommand.Transaction = oTrans;

				daProducts.MissingSchemaAction = MissingSchemaAction.AddWithKey; // necessrio carregar esquema com chave primria
				daProducts.Fill(dsSales, "Products");

				//
				// percorrer a DataTable de detalhes
				//
				foreach(DataRow drDetails in dsSales.Tables["SaleDetails"].Rows)
				{
					//
					// Obter o DataRow do produto com o ID correspondente ao produto a vender
					//
					long lProdID = (int)drDetails["ProductID"];
					DataRow drProd = dsSales.Tables["Products"].Rows.Find(lProdID);

					//
					// determinar preo da venda da quantidade de produto vendido
					//
					drDetails.BeginEdit();

					long lQtd  = (int)drDetails["Quantity"];
					float lPrc = (float)drProd["UnitPrice"];
					drDetails["TotalCost"] = lQtd * lPrc;

					//
					// associar o ID do cabecalho de venda a esta linha de venda
					//
					drDetails["SaleID"] = lSaleID;

					drDetails.EndEdit();
					
					//
					// actualizar stock do produto
					//
					drProd.BeginEdit();
					drProd["StockQtd"] = (int)drProd["StockQtd"] - lQtd;
					drProd.EndEdit();
				}

				//
				// criar o comando 'SQL Update' do DataAdapter para actualizar produtos na base de dados
				//
				OleDbCommandBuilder cbProducts = new OleDbCommandBuilder(daProducts);
				daProducts.UpdateCommand = cbProducts.GetUpdateCommand();

				//
				// criar o DataAdapter para actualizar detalhes da venda na base de dados
				//
				OleDbDataAdapter daSaleDet = new OleDbDataAdapter("SELECT * FROM SaleDetails", oConn);
				daSaleDet.SelectCommand.Transaction = oTrans;
				OleDbCommandBuilder cbSaleDet = new OleDbCommandBuilder(daSaleDet);
				daSaleDet.InsertCommand = cbSaleDet.GetInsertCommand();
				daSaleDet.UpdateCommand = cbSaleDet.GetUpdateCommand();

				//
				// enviar para a base de dados
				//
				daProducts.Update(dsSales, "Products");
				daSaleDet.Update(dsSales, "SaleDetails");

				//
				// fechar transaco
				//
				oTrans.Commit();

				//
				// venda adicionada - aceitar as alteraes no DataSet
				//
				dsSales.AcceptChanges();
			}

			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				if (oTrans != null)
					oTrans.Rollback();

				dsSales.RejectChanges();

				System.Console.WriteLine("EXCEPO no mtodo <AddSale> 'class Sale': " + ex);
				status = ShopStatusEnum.ERROR;
			}

			finally
			{
				//
				// fechar a conexo
				//
				oConn.Close();
			}

			return status;
		}

		//
		// mtodo usado para tratar eventos na insero de registos e obter o cdigo (autonumber) gerado
		// adaptado de:
		// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/cpguide/html/cpconRetrievingIdentityOrAutonumberValues.asp
		//
		protected void Sales_RowUpdated(object sender, OleDbRowUpdatedEventArgs args)
		{
			int newID = 0;

			try
			{
				// Include a variable and a command to retrieve the identity value from the Access database.
				OleDbCommand idCMD = new OleDbCommand("SELECT @@IDENTITY", args.Command.Connection);
				idCMD.Transaction = args.Command.Transaction;

				// Retrieve the identity value and store it in the CategoryID column.
				newID = (int)idCMD.ExecuteScalar();
				args.Row["SaleID"] = newID;
			}

			catch(System.Data.OleDb.OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO 'OLEDB' na 'class Sale': " + ex);
			}

			finally
			{
			}
		}

		public ShopStatusEnum AddShopSale(string sUser, string sPass, long lCustomerID, DateTime dtDate, ShopDataSet dsShop)
		{
			ShopData shop = new ShopData();

			//
			// validar utilizador
			//
			ShopStatusEnum status = Comum.ValidateUser(sUser, sPass);

			if (status != ShopStatusEnum.OK)
			{
				return status;
			}

			//
			// verificar se o cliente existe
			//
			status = Comum.CheckIfExistsID("Customers", "CustomerID", lCustomerID);

			if ( status != ShopStatusEnum.OK)
			{
				status = ShopStatusEnum.INVALID_CUSTOMER_ID;
				return status;
			}

			string sProdList = "(";

			OleDbTransaction oTrans = null;

			try
			{
				//
				// verificar se os produtos colocados nas linhas de venda existem e se esto em stock
				//
				foreach(ShopDataSet.SaleDetailsRow sdr in dsShop.SaleDetails.Rows)
				{
					//
					// obter cdigo de produto e quantidade pretendida do recordset
					//
					long lProdID = sdr.ProductID;
					long lQuant  = sdr.Quantity;

					//
					// verificar a existncia do produto em stock
					//
					status = Product.InStock(lProdID, lQuant);
					if (status != ShopStatusEnum.OK)
						return status;

					sProdList += lProdID.ToString() + ",";
				}

				sProdList = sProdList.Substring(0, sProdList.Length-1) + ")";

				//
				// est tudo OK ==> inserir venda
				//
				int iSaleID;
				status = ShopStatusEnum.OK;

				//
				// criar objecto de conexo  base de dados e abrir a conexo
				//
				shop.ShopConnection.Open();

				//
				// iniciar transaco
				//
				oTrans = shop.ShopConnection.BeginTransaction();

				//
				// associar a transaco aos comandos que vo ser executados
				//
				shop.ShopDASales.InsertCommand.Transaction = oTrans;
				shop.ShopDAProducts.SelectCommand.Transaction = oTrans;
				shop.ShopDAProducts.UpdateCommand.Transaction = oTrans;
				shop.ShopDASaleDetails.InsertCommand.Transaction = oTrans;

				//
				// associar mtodo para tratamento de eventos
				//
				shop.ShopDASales.RowUpdated += new OleDbRowUpdatedEventHandler(this.Sales_RowUpdated);

				//
				// coloca a informao da venda
				//
				ShopDataSet.SalesRow sr = dsShop.Sales.AddSalesRow((int)lCustomerID, dtDate);

				//
				// Insere a nova venda na base de dados
				//
				shop.ShopDASales.Update(dsShop);

				//
				// obter novo cdigo de id gerado pela BD
				//
				iSaleID = (int)sr.SaleID;

				//
				// Carregar a tabela 'Products' para o dataset 'dsSales'
				//
				shop.ShopDAProducts.SelectCommand.CommandText += " WHERE ProductID IN " + sProdList;
				shop.ShopDAProducts.Fill(dsShop);

				//
				// percorrer a DataTable de detalhes
				//
				foreach(ShopDataSet.SaleDetailsRow drDetails in dsShop.SaleDetails.Rows)
				{
					//
					// Obter o DataRow do produto com o ID correspondente ao produto a vender
					//
					ShopDataSet.ProductsRow drProd = (ShopDataSet.ProductsRow)dsShop.Products.Rows.Find(drDetails.ProductID);

					//
					// determinar preo da venda da quantidade de produto vendido
					//
					drDetails.TotalCost = drDetails.Quantity * drProd.UnitPrice;

					//
					// associar o ID do cabecalho de venda a esta linha de venda
					//
					drDetails.SaleID = iSaleID;
					
					//
					// actualizar stock do produto
					//
					drProd.StockQtd += drDetails.Quantity;
				}

				//
				// actualizar produtos na base de dados
				//
				shop.ShopDAProducts.Update(dsShop);

				//
				// actualizar detalhes de venda na base de dados
				//
				shop.ShopDASaleDetails.Update(dsShop);

				//
				// fechar transaco
				//
				oTrans.Commit();

				//
				// venda adicionada - aceitar as alteraes no DataSet para no as considerar no prximo 'Update'
				//
				dsShop.AcceptChanges();
			}

			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				if (oTrans != null)
					oTrans.Rollback();

				dsShop.RejectChanges();

				System.Console.WriteLine("EXCEPO no mtodo <AddSale> 'class Sale': " + ex);
				status = ShopStatusEnum.ERROR;
			}

			finally
			{
				//
				// fechar a conexo
				//
				shop.ShopConnection.Close();

				//
				// dissociar mtodo para tratamento de eventos
				//
				shop.ShopDASales.RowUpdated -= new OleDbRowUpdatedEventHandler(this.Sales_RowUpdated);
			}
			
			return ShopStatusEnum.OK;
		}

	}
}
